@@ -186,7 +186,14 @@ GtkWidget *create_char_table()
 		else
 			row[0] = "";
 		for(col = 1; col < 4; col++) {
+#if defined(__GNUC__) && (__GNUC__ > 4)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
 			label = g_strdup_printf(fmt[col], i);
+#if defined(__GNUC__) && (__GNUC__ > 4)
+#pragma GCC diagnostic pop
+#endif
 			row[col] = label;
 		}
 		for(col = 0; col < 8; col++) {
@@ -226,10 +233,10 @@ GtkWidget *create_char_table()
 	lbl = gtk_label_new ("");
 	gtk_widget_show(lbl);
 
-	vbox = gtk_vbox_new(FALSE, 4);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 4);
 	gtk_widget_show(vbox);
 
-	hbox = gtk_hbox_new (FALSE, 4);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
 	gtk_widget_show(hbox);
 
 	gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
