@@ -53,8 +53,15 @@ enum {
 	LAST_SIGNAL
 };
 
-enum {
+enum ClipboardTargets {
+  TARGET_HEXDATA,
   TARGET_STRING,
+  N_TARGETS
+};
+
+static const GtkTargetEntry clipboard_targets[] = {
+	{ "HEXDATA", 0, TARGET_HEXDATA },
+	{ "STRING", 0, TARGET_STRING }
 };
 
 struct _GtkHex_AutoHighlight
@@ -80,6 +87,9 @@ struct _GtkHexPrivate
 
 	gint default_cpl;
 	gint default_lines;
+
+	guchar *clip_buf;
+	gint clip_buf_len;
 };
 
 static gint gtkhex_signals[LAST_SIGNAL] = { 0 };
@@ -906,9 +916,9 @@ draw_shadow (GtkWidget *widget,
 		                  cr,
 		                  border,
 		                  border,
-		                  8 * gh->char_width + padding.left + padding.right,
+		                  9 * gh->char_width + padding.left + padding.right,
 		                  allocation.height - 2 * border);
-		x += 8 * gh->char_width + padding.left + padding.right + gh->extra_width/2;
+		x += 9 * gh->char_width + padding.left + padding.right + gh->extra_width/2;
 	}
 
 	gtk_render_frame (context,
@@ -964,7 +974,7 @@ static void recalc_displays(GtkHex *gh, guint width, guint height) {
 	               2 * padding.left + 2 * padding.right + req.width;
 
 	if(gh->show_offsets)
-		total_width -= padding.left + padding.right + 8 * gh->char_width;
+		total_width -= padding.left + padding.right + 9 * gh->char_width;
 
 	total_cpl = total_width / gh->char_width;
 
@@ -1373,6 +1383,45 @@ static void bytes_changed(GtkHex *gh, gint start, gint end)
     }
 }
 
+static void
+clipboard_get_cb (GtkClipboard *clipboard,
+		GtkSelectionData *selection_data,
+		guint info,
+		gpointer user_data)
+{
+	GtkHex *gh = GTK_HEX(user_data);
+
+	switch (info)
+	{
+		case TARGET_HEXDATA:
+			if (gh->priv->clip_buf)
+			{
+				gtk_selection_data_set (selection_data,
+						gdk_atom_intern ("HEXDATA", FALSE),
+						8,	/* ie, 8-bit characters */
+						gh->priv->clip_buf,
+						gh->priv->clip_buf_len);
+			}
+			break;
+
+		case TARGET_STRING:
+			if (gh->priv->clip_buf)
+			{
+				gtk_selection_data_set (selection_data,
+						GDK_SELECTION_TYPE_STRING,
+						8,	/* ie, 8-bit characters */
+						gh->priv->clip_buf,
+						gh->priv->clip_buf_len);
+			}
+			break;
+
+		default:
+			g_critical ("Invalid clipboard data.");
+			return;
+	}
+
+}
+
 static void primary_get_cb(GtkClipboard *clipboard,
 						   GtkSelectionData *data, guint info,
 						   gpointer user_data) {
@@ -1392,21 +1441,12 @@ static void primary_get_cb(GtkClipboard *clipboard,
 	}
 }
 
-static void primary_clear_cb(GtkClipboard *clipboard,
-							 gpointer user_data_or_owner) {
-}
-
 void gtk_hex_set_selection(GtkHex *gh, gint start, gint end)
 {
 	gint length = gh->document->file_size;
 	gint oe, os, ne, ns;
 	GtkHexClass *klass = GTK_HEX_CLASS(GTK_WIDGET_GET_CLASS(gh));
 
-	static const GtkTargetEntry targets[] = {
-		{ "STRING", 0, TARGET_STRING }
-	};
-	static const gint n_targets = sizeof(targets) / sizeof(targets[0]);
-
 	if (end < 0)
 		end = length;
 
@@ -1435,8 +1475,8 @@ void gtk_hex_set_selection(GtkHex *gh, gint start, gint end)
 	}
 
 	if(gh->selection.start != gh->selection.end)
-		gtk_clipboard_set_with_data(klass->primary, targets, n_targets,
-									primary_get_cb, primary_clear_cb,
+		gtk_clipboard_set_with_data(klass->primary, clipboard_targets, N_TARGETS,
+									primary_get_cb, NULL,
 									gh);
 }
 
@@ -1468,19 +1508,18 @@ void gtk_hex_clear_selection(GtkHex *gh)
 
 void gtk_hex_delete_selection(GtkHex *gh)
 {
-	guint start;
-	guint end;
+	guint start, end, len;
 
 	start = MIN(gh->selection.start, gh->selection.end);
 	end = MAX(gh->selection.start, gh->selection.end);
 
+	len = end - start + 1;
+	g_assert (len);
+
 	gtk_hex_set_selection(gh, 0, 0);
 
-	if(start != end) {
-		if(start < gh->cursor_pos)
-			gtk_hex_set_cursor(gh, gh->cursor_pos - end + start);
-		hex_document_delete_data(gh->document, MIN(start, end), end - start, TRUE);
-	}
+	hex_document_delete_data(gh->document, MIN(start, end), len, TRUE);
+	gtk_hex_set_cursor(gh, start);
 }
 
 static void gtk_hex_validate_highlight(GtkHex *gh, GtkHex_Highlight *hl)
@@ -1684,19 +1723,30 @@ void gtk_hex_paste_from_clipboard(GtkHex *gh)
 
 static void gtk_hex_real_copy_to_clipboard(GtkHex *gh)
 {
-	gint start_pos; 
-	gint end_pos;
 	GtkHexClass *klass = GTK_HEX_CLASS(GTK_WIDGET_GET_CLASS(gh));
 
+	gint start_pos, end_pos, len;
+
 	start_pos = MIN(gh->selection.start, gh->selection.end);
 	end_pos = MAX(gh->selection.start, gh->selection.end);
- 
-	if(start_pos != end_pos) {
-		guchar *text = hex_document_get_data(gh->document, start_pos,
-											 end_pos - start_pos);
-		gtk_clipboard_set_text(klass->clipboard, text, end_pos - start_pos);
-		g_free(text);
-	}
+
+	/* +1 because we're counting the number of characters to grab here.
+	 * You have to actually include the first character in the range.
+	 */
+	len = end_pos - start_pos + 1;
+	g_return_if_fail (len);
+
+	g_clear_pointer (&gh->priv->clip_buf, g_free);
+	gh->priv->clip_buf = hex_document_get_data (gh->document,
+			start_pos, len);
+	gh->priv->clip_buf_len = len;
+
+	gtk_clipboard_set_with_data (klass->clipboard,
+			clipboard_targets,
+			N_TARGETS,
+			clipboard_get_cb,
+			NULL,
+			gh);
 }
 
 static void gtk_hex_real_cut_to_clipboard(GtkHex *gh)
@@ -1710,15 +1760,37 @@ static void gtk_hex_real_cut_to_clipboard(GtkHex *gh)
 static void gtk_hex_real_paste_from_clipboard(GtkHex *gh)
 {
 	GtkHexClass *klass = GTK_HEX_CLASS(GTK_WIDGET_GET_CLASS(gh));
-	gchar *text;
+	GtkSelectionData *selection_data = NULL;
+	const guchar *text = NULL;
+	GdkAtom hex_atom = gdk_atom_intern ("HEXDATA", FALSE);
+	int len;
 
-	text = gtk_clipboard_wait_for_text(klass->clipboard);
-	if(text) {
-		hex_document_set_data(gh->document, gh->cursor_pos,
-							  strlen(text), 0, text, TRUE);
-		gtk_hex_set_cursor(gh, gh->cursor_pos + strlen(text));
-		g_free(text);
+	if (gtk_clipboard_wait_is_target_available (klass->clipboard,
+				hex_atom))
+	{
+		selection_data = gtk_clipboard_wait_for_contents (
+				klass->clipboard, hex_atom);
 	}
+	else
+	{
+		selection_data = gtk_clipboard_wait_for_contents (
+				klass->clipboard, GDK_TARGET_STRING);
+	}
+
+	if (selection_data)
+	{
+		text = gtk_selection_data_get_data (selection_data);
+		len = gtk_selection_data_get_length (selection_data);
+	}
+
+	if (text && len)
+	{
+		hex_document_set_data(gh->document, gh->cursor_pos,
+				len, 0, text, TRUE);
+		gtk_hex_set_cursor(gh, gh->cursor_pos + len);
+	}
+
+	g_clear_pointer (&selection_data, gtk_selection_data_free);
 }
 
 static void gtk_hex_finalize(GObject *o) {
@@ -1969,7 +2041,7 @@ static void gtk_hex_size_allocate(GtkWidget *w, GtkAllocation *alloc) {
 	my_alloc.y = border_width + padding.top;
 	my_alloc.height = MAX (alloc->height - 2 * border_width - padding.top - padding.bottom, 1);
 	if(gh->show_offsets) {
-		my_alloc.width = 8*gh->char_width;
+		my_alloc.width = 9*gh->char_width;
 		gtk_widget_size_allocate(gh->offsets, &my_alloc);
 		gtk_widget_queue_draw(gh->offsets);
 		my_alloc.x += padding.left + padding.right + my_alloc.width + gh->extra_width/2;
@@ -2028,7 +2100,7 @@ static void gtk_hex_size_request(GtkWidget *w, GtkRequisition *req) {
 		sb_req.width + gh->char_width * (gh->priv->default_cpl + (gh->priv->default_cpl - 1) /
 										 gh->group_type);
 	if(gh->show_offsets)
-		req->width += padding.left + padding.right + 8 * gh->char_width;
+		req->width += padding.left + padding.right + 9 * gh->char_width;
 	req->height = gh->priv->default_lines * gh->char_height + padding.top + padding.bottom +
 		2*gtk_container_get_border_width(GTK_CONTAINER(w));
 }
