@@ -176,8 +176,8 @@ about_cb (GtkAction  *action,
 	                                NULL);
 
 	/* Translators: these two strings here indicate the copyright time span,
-	   e.g. 1998-2012. */
-	copyright = g_strdup_printf (_("Copyright © %Id–%Id The GHex authors"), 1998, 2012);
+	   e.g. 1998-2018. */
+	copyright = g_strdup_printf (_("Copyright © %Id–%Id The GHex authors"), 1998, 2018);
 
 	gtk_show_about_dialog (GTK_WINDOW (window),
 	                       "authors", authors,
@@ -185,12 +185,12 @@ about_cb (GtkAction  *action,
 	                       "copyright", copyright,
 	                       "documenters", documentation_credits,
 	                       "license", license_translated,
-	                       "logo-icon-name", PACKAGE,
+	                       "logo-icon-name", "org.gnome.GHex",
 	                       "program-name", "GHex",
 	                       "title", _("About GHex"),
 	                       "translator-credits", _("translator-credits"),
 	                       "version", PACKAGE_VERSION,
-	                       "website", "http://live.gnome.org/Ghex",
+	                       "website", "https://wiki.gnome.org/Apps/Ghex",
 	                       "website-label", _("GHex Website"),
 	                       "wrap-license", TRUE,
 	                       NULL);
